"""Re-export from galileo.shared.base — will be deprecated once all __future__ modules are migrated."""

from galileo.shared.base import StateManagementMixin, SyncState

__all__ = ["StateManagementMixin", "SyncState"]
