"""Re-export from galileo.collaborator — will be deprecated once all __future__ modules are migrated."""

from galileo.collaborator import Collaborator, CollaboratorRole

__all__ = ["Collaborator", "CollaboratorRole"]
